@@ -40,6 +40,8 @@ class CodeConvectionFormatter:
             while len(snake_separation[-1]) == 0:
                 snake_separation = snake_separation[:-1]
             return snake_separation
+        elif content.upper() == content:
+            return [content.lower()]
         else:
             camel_separation = []
             name_part = str()
@@ -61,7 +63,7 @@ class CodeConvectionFormatter:
             new_token_content += separated_content[i]
             if i < len(separated_content) - 1 or (class_field and separated_content[i] != '_'):
                 new_token_content += '_'
-        return lexer.Token(new_token_content, lexer.TokenType.identifier)
+        return lexer.Token(new_token_content, lexer.TokenType.identifier, token.line(), token.column())
 
     @classmethod
     def to_camel_case(cls, token, pascal_case=False, constant=False):
@@ -80,7 +82,20 @@ class CodeConvectionFormatter:
             s[0] = s[0].upper()
             s = ''.join(s)
             new_token_content += s
-        return lexer.Token(new_token_content, lexer.TokenType.identifier)
+        return lexer.Token(new_token_content, lexer.TokenType.identifier, token.line(), token.column())
+
+    def get_initial_comment(self, file_path):
+        return "// {}\n".format(os.path.basename(file_path))
+
+    def get_declaration_comment(self, type_class, type_name):
+        return '/// Description for {} {}\n'.format(type_class, type_name)
+
+    def get_type_description_comment(self, type_class, type_name):
+        return '/// Description for {} {}\n'.format(type_class, type_name)
+
+    def get_function_description_comment(self, func_name):
+        return '/// Description for function {}\n'.format(type, func_name)
+
 
     def format_identifier(self, token, id_type=IdType.Variable):
         if id_type == IdType.Type or id_type == IdType.Enum:
@@ -133,6 +148,27 @@ class CodeConvectionFormatter:
         else:
             return False
 
+    def check_if_declaration(self, tokens, i):
+        i -= 1
+        res = False
+        namespace_member = False
+        while True:
+            if i > 0 and tokens[i].type() == lexer.TokenType.whitespace:
+                i -= 1
+            elif i > 0 and tokens[i].content() == '::':
+                namespace_member = True
+                i -= 1
+            elif namespace_member and i > 0 and tokens[i].type() == lexer.TokenType.identifier:
+                namespace_member = False
+                i -= 1
+            else:
+                break
+        if tokens[i].type() != lexer.TokenType.whitespace:
+            if tokens[i].is_type() or tokens[i].type() == lexer.TokenType.identifier:
+                res = True
+        return res
+
+
     def check_if_called(self, tokens, i):
         return tokens[i - 1].content() == '.'
 
@@ -148,33 +184,43 @@ class CodeConvectionFormatter:
         next_is_const = False
         next_is_class = False
         next_class_name = str()
+
         next_is_struct = False
         next_is_enum = False
         next_is_namespace = False
-        next_is_class_member = False
+        next_is_class_instance = False
+        next_is_ignored = False
         class_method = False
         prev_class_name = None
-        is_main = False
+        next_is_tempalte = False
+        template_count = 0
+        template_typenames = list()
+        next_is_function_declaration = False
+
         while i < len(tokens) and tokens[i].content() != '}':
             content = tokens[i].content()
             if tokens[i].type() == lexer.TokenType.identifier:
                 is_function = self.check_if_function(tokens, i)
                 is_called = self.check_if_called(tokens, i)
+                is_declaration = self.check_if_declaration(tokens, i)
                 is_class = content in local_variable_dictionary['class']['names']
                 is_struct = content in local_variable_dictionary['struct']['names']
                 is_enum = content in local_variable_dictionary['enum']['names']
                 is_macro = content in local_variable_dictionary['macro']['names']
                 is_namespace = content in local_variable_dictionary['namespace']['names']
+
                 token_type = None
 
-                if next_is_class_member and not is_function:
+                if next_is_class_instance and not is_function:
                     local_variable_dictionary['class']['variables'].append(content)
-                    next_is_class_member = False
+                    next_is_class_instance = False
 
-                if is_class:
+                if next_is_ignored:
+                    pass
+                elif is_class:
                     token_type = IdType.Type
                     if not is_function:
-                        next_is_class_member = True
+                        next_is_class_instance = True
                         prev_class_name = content
                         if content not in class_members:
                             class_members[content] = set()
@@ -187,12 +233,14 @@ class CodeConvectionFormatter:
                 elif is_namespace:
                     token_type = IdType.Namespace
                 elif is_function:
-                    if next_is_class_member:
-                        next_is_class_member = False
+                    if next_is_class_instance:
+                        next_is_class_instance = False
                         class_method = True
                     if content == 'main':
                         token_type = IdType.Main
                     else:
+                        if is_declaration:
+                            next_is_function_declaration = True
                         token_type = IdType.Function
                 else:
                     if next_is_const:
@@ -203,12 +251,18 @@ class CodeConvectionFormatter:
                         local_variable_dictionary['enum']['names'].append(content)
                     elif next_is_class:
                         token_type = IdType.Type
-                        next_class_name = content
-                        class_members[content] = set()
-                        local_variable_dictionary['class']['names'].append(content)
+                        if template_count > 0:
+                            template_typenames.append(content)
+                        else:
+                            next_class_name = content
+                            class_members[content] = set()
+                            local_variable_dictionary['class']['names'].append(content)
                     elif next_is_struct:
                         token_type = IdType.Type
-                        local_variable_dictionary['struct']['names'].append(content)
+                        if template_count > 0:
+                            template_typenames.append(content)
+                        else:
+                            local_variable_dictionary['struct']['names'].append(content)
                     elif next_is_namespace:
                         token_type = IdType.Namespace
                         local_variable_dictionary['namespace']['names'].append(content)
@@ -260,7 +314,7 @@ class CodeConvectionFormatter:
 
                 if content == 'const':
                     next_is_const = True
-                elif content == 'class':
+                elif content == 'class' or content == 'typename':
                     next_is_class = True
                 elif content == 'struct':
                     next_is_struct = True
@@ -268,6 +322,15 @@ class CodeConvectionFormatter:
                     next_is_enum = True
                 elif content == 'namespace':
                     next_is_namespace = True
+                elif content == 'template':
+                    template_count += 1
+                    next_is_tempalte = True
+                elif tokens[i].is_type() and template_count > 0:
+                    next_is_class = True
+                elif content == '>' and template_count > 0:
+                    template_count -= 1
+                elif content == '::' and not next_is_class_instance:
+                    next_is_ignored = True
                 elif content == '{':
 
                     next_scope = {'type': 'block', 'name': current_scope['name']}
@@ -287,56 +350,163 @@ class CodeConvectionFormatter:
                         next_scope['type'] = 'class method'
                         next_scope['name'] = prev_class_name
 
+                    next_variable_dictionary = copy.deepcopy(local_variable_dictionary)
+                    if next_is_tempalte:
+                        for typename in template_typenames:
+                            next_variable_dictionary['class']['names'].append(typename)
+                            next_is_tempalte = False
+
                     if current_scope['type'] == 'class' and next_scope['type'] == 'block':
-                        next_scope['type'] = 'class'
+                        next_scope['type'] = 'class method'
                     formatted_tokens.append(tokens[i])
-                    i, formatted_scope, q, w = self.format_scope(tokens, i + 1, next_scope, local_variable_dictionary,
+                    i, formatted_scope, q, w = self.format_scope(tokens, i + 1, next_scope, next_variable_dictionary,
                                                            class_members)
                     formatted_tokens += formatted_scope
                 formatted_tokens.append(tokens[i])
             i += 1
         return i, formatted_tokens, local_variable_dictionary, class_members
 
+    def add_documentation(self, tokens, file_path):
+        i = 0
+
+        while len(tokens) > 0 and tokens[0].type() == lexer.TokenType.whitespace:
+            tokens = tokens[1:]
+
+        initial_comment = self.get_initial_comment(file_path)
+        if tokens[0].content() != initial_comment:
+            comment = lexer.Token(initial_comment, lexer.TokenType.comment, 0, 0, True)
+            comment.set_error_message('No file description')
+            tokens.insert(0, comment)
+
+        comment_line = -1
+        next_is_declaration = False
+        type_class = None
+        is_template = False
+        template_count = 0
+        name = str()
+        indent = 0
+        indent_stack = []
+        while i < len(tokens):
+            content = tokens[i].content()
+            if content == 'template':
+                comment_line = tokens[i].line()-1
+                is_template = True
+                template_count += 1
+            elif template_count > 0:
+                if content == '>':
+                    template_count -= 1
+            elif content == 'class':
+                next_is_declaration = True
+                type_class = 'class'
+                indent = tokens[i].column()
+            elif content == 'struct':
+                next_is_declaration = True
+                type_class = 'struct'
+                indent = tokens[i].column()
+            elif content == 'enum':
+                next_is_declaration = True
+                type_class = 'enum'
+                indent = tokens[i].column()
+            elif tokens[i].type() == lexer.TokenType.identifier:
+                is_function = self.check_if_function(tokens, i)
+                is_declaration = self.check_if_declaration(tokens, i)
+                if next_is_declaration:
+                    if not is_template:
+                        comment_line = tokens[i].line()-1
+                    name = tokens[i].content()
+                    next_is_declaration = False
+                elif is_function and is_declaration:
+                    if not is_template:
+                        comment_line = tokens[i].line()-1
+                    type_class = 'function'
+                    name = content
+
+                    j = i
+                    while tokens[j].column() > 1:
+                        j -= 1
+                    while tokens[j].type() == lexer.TokenType.whitespace:
+                        indent_stack += [tokens[j]]
+                        j += 1
+
+            elif content in ['{', ';']:
+                if name != '':
+                    j = i
+                    while j > 0 and tokens[j].line() > comment_line:
+                        j -= 1
+                    j1 = j
+                    while j1 > 0 and tokens[j1].type() == lexer.TokenType.whitespace:
+                        j1 -= 1
+                    if tokens[j1].type() != lexer.TokenType.comment or (tokens[j1].type() == lexer.TokenType.comment and len(tokens[j1].get_error_message()) != 0):
+                        declaration_comment = self.get_declaration_comment(type_class, name)
+                        comment_token = lexer.Token(declaration_comment, lexer.TokenType.comment, comment_line, indent+1, True)
+                        comment_token.set_error_message('No description of {}'.format(name))
+                        tokens.insert(j + 1, comment_token)
+                        for k in range(indent-1):
+                            generated_token = lexer.Token(' ', lexer.TokenType.whitespace, comment_line, k, True)
+                            tokens.insert(j + 1, generated_token)
+                            i += 1
+                        indent = 1
+                        next_column = 1
+                        for k in range(len(indent_stack)):
+                            indent_content = indent_stack[k].content()
+                            new_token = lexer.Token(indent_content, lexer.TokenType.whitespace, comment_line, next_column, True)
+                            tokens.insert(j+1, new_token)
+                            i += 1
+                            next_column += len(indent_content)
+                        indent_stack = []
+                        name = ''
+                        is_template = False
+                        i += 1
+
+            i += 1
+
     def format_file(self, file_path, format_file=False, project_formatting=False, in_file_call=False):
         with open(file_path, 'r') as file_reader:
             file = file_reader.read()
-        tokens = lexer.lex(file)
-        current_scope = {'type': 'file', 'name': file_path}
-        empty_config = {'names': [], 'variables': []}
-        variable_dictionary = {'class': copy.deepcopy(empty_config),
-                               'struct': copy.deepcopy(empty_config),
-                               'enum': copy.deepcopy(empty_config),
-                               'macro': copy.deepcopy(empty_config),
-                               'namespace': copy.deepcopy(empty_config),
-                               'file_name': file_path}
-        class_members = dict()
-        i, formatted_tokens, variable_dictionary, class_members = self.format_scope(tokens, 0, current_scope, variable_dictionary, class_members)
-        if not in_file_call:
-            error_id = 1
-            with open(file_path+'_verification.log', 'w') as log_writer:
-                if file_path != self.normalize_name(file_path):
-                    log_writer.write('{id}. {path}: wrong file extension\n'.format(id=error_id, path=file_path))
-                    error_id += 1
-                for i in range(len(tokens)):
-                    if tokens[i].content() != formatted_tokens[i].content():
-                        error_message = '{id}. {path}: line {line} - {content}: {error_message}\n'.format(
-                            id=error_id, path=file_path, line = tokens[i].line(), content=tokens[i].content(),
-                            error_message=tokens[i].get_error_message()
-                        )
-                        log_writer.write(error_message)
+        try:
+            tokens = lexer.lex(file)
+            current_scope = {'type': 'file', 'name': file_path}
+            empty_config = {'names': [], 'variables': []}
+            variable_dictionary = {'class': copy.deepcopy(empty_config),
+                                   'struct': copy.deepcopy(empty_config),
+                                   'enum': copy.deepcopy(empty_config),
+                                   'macro': copy.deepcopy(empty_config),
+                                   'namespace': copy.deepcopy(empty_config),
+                                   'file_name': file_path}
+            class_members = dict()
+            i, formatted_tokens, variable_dictionary, class_members = self.format_scope(tokens, 0, current_scope, variable_dictionary, class_members)
+            self.add_documentation(formatted_tokens, file_path)
+            if not in_file_call:
+                error_id = 1
+                with open(file_path+'_verification.log', 'w') as log_writer:
+                    if file_path != self.normalize_name(file_path):
+                        log_writer.write('{id}. {path}: wrong file extension\n'.format(id=error_id, path=file_path))
                         error_id += 1
-        if format_file:
-            if project_formatting:
-                self.files[file_path] = str()
-                for token in formatted_tokens:
-                    self.files[file_path] += token.content()
-            else:
-                with open(file_path, 'w') as file_writer:
+                    shift = 0
+                    for i in range(len(tokens)):
+                        while formatted_tokens[i+shift].is_generated():
+                            shift += 1
+                        if tokens[i].content() != formatted_tokens[i+shift].content():
+                            error_message = '{id}. {path}: line {line} - {content}: {error_message}\n'.format(
+                                id=error_id, path=file_path, line = tokens[i].line(), content=tokens[i].content(),
+                                error_message=tokens[i].get_error_message()
+                            )
+                            log_writer.write(error_message)
+                            error_id += 1
+            if format_file:
+                if project_formatting:
+                    self.files[file_path] = str()
                     for token in formatted_tokens:
-                        file_writer.write(token.content())
-                os.rename(file_path, self.normalize_name(file_path))
+                        self.files[file_path] += token.content()
+                else:
+                    with open(file_path, 'w') as file_writer:
+                        for token in formatted_tokens:
+                            file_writer.write(token.content())
+                    os.rename(file_path, self.normalize_name(file_path))
 
-        return variable_dictionary, class_members
+            return variable_dictionary, class_members
+        except Exception as e:
+            print(e)
 
     @classmethod
     def is_cpp_file(cls, file_name):
